##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pagelet Content Providers
"""
import zope.component
import zope.interface
import zope.publisher.interfaces.http

import z3c.pagelet.interfaces


@zope.interface.implementer(z3c.pagelet.interfaces.IPageletRenderer)
class PageletRenderer:
    """Render the adapted pagelet."""

    zope.component.adapts(zope.interface.Interface,
                          zope.publisher.interfaces.http.IHTTPRequest,
                          z3c.pagelet.interfaces.IPagelet)

    def __init__(self, context, request, pagelet):
        self.__updated = False
        self.__parent__ = pagelet
        self.context = context
        self.request = request

    def update(self):
        pass

    def render(self):
        return self.__parent__.render()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests
"""
import doctest
import itertools
import unittest

import zope.component
import zope.formlib.exception
import zope.formlib.interfaces
import zope.formlib.textwidgets
import zope.schema
import zope.traversing.adapters
from zope.configuration import xmlconfig
from zope.formlib import form
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.site.testing import siteSetUp
from zope.site.testing import siteTearDown

from z3c.pagelet import outputchecker


checker = outputchecker.OutputChecker(patterns=[])


def setUp(test):
    root = siteSetUp(True)
    test.globs['root'] = root

    zope.component.provideAdapter(
        zope.traversing.adapters.DefaultTraversable,
        [None],
    )

    # setup widgets
    zope.component.provideAdapter(
        zope.formlib.textwidgets.TextWidget,
        [zope.schema.interfaces.ITextLine, IBrowserRequest],
        zope.formlib.interfaces.IInputWidget)

    zope.component.provideAdapter(
        zope.formlib.exception.WidgetInputErrorView,
        [zope.formlib.interfaces.IWidgetInputError,
         zope.publisher.interfaces.browser.IBrowserRequest,
         ],
        zope.formlib.interfaces.IWidgetInputErrorView,
    )
    zope.component.provideAdapter(
        zope.formlib.widget.UnicodeDisplayWidget,
        [zope.schema.interfaces.ITextLine,
         zope.publisher.interfaces.browser.IBrowserRequest,
         ],
        zope.formlib.interfaces.IDisplayWidget,
    )
    zope.component.provideAdapter(form.render_submit_button, name='render')


def setUpZPT(test):
    setUp(test)

    # register provider TALES
    from zope.browserpage import metaconfigure
    from zope.contentprovider import tales
    metaconfigure.registerType('provider', tales.TALESProviderExpression)


def setUpZ3CPT(suite):
    setUp(suite)
    import z3c.pt
    import z3c.ptcompat
    xmlconfig.XMLConfig('configure.zcml', z3c.pt)()
    xmlconfig.XMLConfig('configure.zcml', z3c.ptcompat)()


def tearDown(test):
    siteTearDown()


def test_suite():
    flags = (
        doctest.NORMALIZE_WHITESPACE
        | doctest.ELLIPSIS
        | doctest.IGNORE_EXCEPTION_DETAIL
    )
    tests = ((
        doctest.DocFileSuite(
            'README.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=flags, checker=checker),
        doctest.DocFileSuite(
            'zcml.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=flags, checker=checker),
    ) for setUp in (setUpZPT, setUpZ3CPT, ))

    return unittest.TestSuite(itertools.chain(*tests))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pagelet mixin classes
"""
import zope.component
import zope.interface
from z3c.template.interfaces import IContentTemplate
from z3c.template.interfaces import ILayoutTemplate
from zope.publisher import browser

from z3c.pagelet import interfaces


REDIRECT_STATUS_CODES = (301, 302, 303)


# default pagelet base implementation
@zope.interface.implementer(interfaces.IPagelet)
class BrowserPagelet(browser.BrowserPage):
    """Content generating pagelet with layout template support."""

    template = None
    layout = None

    def update(self):
        pass

    def render(self):
        # render content template
        if self.template is None:
            template = zope.component.queryMultiAdapter(
                (self, self.request, self.context), IContentTemplate)
            if template is None:
                template = zope.component.getMultiAdapter(
                    (self, self.request), IContentTemplate)
            return template(self)
        return self.template()

    def __call__(self):
        """Call update and returns the layout template which calls render."""
        self.update()

        if self.request.response.getStatus() in REDIRECT_STATUS_CODES:
            # don't bother rendering when redirecting
            return ''

        if self.layout is None:
            layout = zope.component.queryMultiAdapter(
                (self, self.request, self.context), ILayoutTemplate)
            if layout is None:
                layout = zope.component.getMultiAdapter(
                    (self, self.request), ILayoutTemplate)
            return layout(self)
        return self.layout()


try:
    from zope.formlib import form
except ModuleNotFoundError:
    pass
else:
    # formlib based pagelet mixin classes
    @zope.interface.implementer(interfaces.IPageletForm)
    class PageletForm(form.FormBase, BrowserPagelet):
        """Form mixin for pagelet implementations."""

        template = None
        layout = None

        __init__ = BrowserPagelet.__init__

        __call__ = BrowserPagelet.__call__

        def render(self):
            # if the form has been updated, it will already have a result
            if self.form_result is None:
                if self.form_reset:
                    # we reset, in case data has changed in a way that
                    # causes the widgets to have different data
                    self.resetForm()
                    self.form_reset = False
                if self.template is None:
                    template = zope.component.queryMultiAdapter(
                        (self, self.request, self.context), IContentTemplate)
                    if template is None:
                        template = zope.component.getMultiAdapter(
                            (self, self.request), IContentTemplate)
                    self.form_result = template(self)
                else:
                    self.form_result = self.template()

            return self.form_result

    @zope.interface.implementer(interfaces.IPageletAddForm)
    class PageletAddForm(PageletForm, form.AddFormBase):
        """Add form mixin for pagelet implementations."""

        def render(self):
            if self._finished_add:
                self.request.response.redirect(self.nextURL())
                return ""
            # render content template
            if self.template is None:
                template = zope.component.queryMultiAdapter(
                    (self, self.request, self.context), IContentTemplate)
                if template is None:
                    template = zope.component.getMultiAdapter(
                        (self, self.request), IContentTemplate)
                return template(self)
            return self.template()

    @zope.interface.implementer(interfaces.IPageletEditForm)
    class PageletEditForm(PageletForm, form.EditFormBase):
        """Edit form mixin for pagelet implementations."""

    @zope.interface.implementer(interfaces.IPageletDisplayForm)
    class PageletDisplayForm(PageletForm, form.DisplayFormBase):
        """Display form mixin for pagelet implementations."""



##############################################################################
#
# Copyright (c) 2007-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup"""

import os

from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


tests_require = [
    "zope.formlib",
    "zope.site",
    "zope.testing",
    "zope.testrunner >= 6.4",
    "zope.traversing",
    "lxml",
    "z3c.pt >= 2.1",
    "z3c.ptcompat >= 1.0",
]


setup(
    name="z3c.pagelet",
    version="4.0",
    author="Roger Ineichen and the Zope Community",
    author_email="zope-dev@zope.dev",
    description="Pagelets are way to specify a template without the O-wrap.",
    long_description=(
        read("README.rst")
        + "\n\n"
        + read("src", "z3c", "pagelet", "README.rst")
        + "\n\n"
        + read("CHANGES.rst")
    ),
    license="ZPL-2.1",
    keywords="zope3 template pagelet layout zpt pagetemplate",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: Implementation :: CPython",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP",
        "Framework :: Zope :: 3",
    ],
    url="https://github.com/zopefoundation/z3c.pagelet",
    python_requires=">=3.9",
    extras_require=dict(
        test=tests_require,
        chameleon=[
            "z3c.pt >= 2.1",
            "z3c.ptcompat",
        ],
        forms=["zope.formlib"],
        docs=["z3c.recipe.sphinxdoc"],
    ),
    install_requires=[
        "setuptools",
        "z3c.template>=1.2.0",
        "zope.browserpage",
        "zope.component>=3.7.0",
        "zope.configuration",
        "zope.contentprovider",
        "zope.interface",
        "zope.publisher",
        "zope.schema",
        "zope.security",
    ],
    include_package_data=True,
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces
"""
import zope.contentprovider.interfaces
from zope.publisher.interfaces.browser import IBrowserPage


class IPageletRenderer(zope.contentprovider.interfaces.IContentProvider):
    """Renders a pagelet by calling it's render method."""


class IPagelet(IBrowserPage):
    """The pagelet."""

    def update():
        """Update the pagelet data."""

    def render():
        """Render the pagelet content w/o o-wrap."""


class IPageletForm(IPagelet):
    """Form mixin for pagelet implementation."""


class IPageletAddForm(IPageletForm):
    """Add form mixin for pagelet implementation."""


class IPageletEditForm(IPageletForm):
    """Edit form mixin for pagelet implementation."""


class IPageletDisplayForm(IPageletForm):
    """FDisplay frm mixin for pagelet implementation."""



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""ZCML Directives
"""
import zope.component
import zope.component.zcml
import zope.configuration.fields
import zope.interface
import zope.schema
import zope.security.checker
import zope.security.zcml
from zope.browserpage import metaconfigure as viewmeta
from zope.configuration.exceptions import ConfigurationError
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from z3c.pagelet import browser
from z3c.pagelet import interfaces


class IPageletDirective(zope.component.zcml.IBasicViewInformation):
    """A directive to register a new pagelet.

    The pagelet directive also supports an undefined set of keyword arguments
    that are set as attributes on the pagelet after creation.
    """

    name = zope.schema.TextLine(
        title="The name of the pagelet.",
        description="The name shows up in URLs/paths. For example 'foo'.",
        required=True)

    class_ = zope.configuration.fields.GlobalObject(
        title="Class",
        description="A class that provides attributes used by the pagelet.",
        required=True,
    )

    permission = zope.security.zcml.Permission(
        title="Permission",
        description="The permission needed to use the pagelet.",
        required=True
    )

    layer = zope.configuration.fields.GlobalObject(
        title="The request interface or class this pagelet is for.",
        description=(
            "Defaults to"
            " zope.publisher.interfaces.browser.IDefaultBrowserLayer."),
        required=False
    )

    for_ = zope.configuration.fields.GlobalObject(
        title="Context",
        description="The content interface or class this pagelet is for.",
        required=False
    )

    provides = zope.configuration.fields.GlobalInterface(
        title="The interface this pagelets provides.",
        description="""
        A pagelet can provide an interface.  This would be used for
        views that support other views.""",
        required=False,
        default=interfaces.IPagelet,
    )


# Arbitrary keys and values are allowed to be passed to the pagelet.
IPageletDirective.setTaggedValue('keyword_arguments', True)


# pagelet directive
def pageletDirective(
        _context, class_, name, permission, for_=zope.interface.Interface,
        layer=IDefaultBrowserLayer, provides=interfaces.IPagelet,
        allowed_interface=None, allowed_attributes=None, **kwargs):

    # Security map dictionary
    required = {}

    # Get the permission; mainly to correctly handle CheckerPublic.
    permission = viewmeta._handle_permission(_context, permission)

    ifaces = list(zope.interface.Declaration(provides).flattened())
    if interfaces.IPagelet not in ifaces:
        raise ConfigurationError("Provides interface must inherit IPagelet.")

    # Build a new class that we can use different permission settings if we
    # use the class more then once.
    cdict = {}
    cdict['__name__'] = name
    cdict.update(kwargs)
    new_class = type(class_.__name__, (class_, browser.BrowserPagelet), cdict)

    # Set up permission mapping for various accessible attributes
    viewmeta._handle_allowed_interface(
        _context, allowed_interface, permission, required)
    viewmeta._handle_allowed_attributes(
        _context, allowed_attributes, permission, required)
    viewmeta._handle_allowed_attributes(
        _context, kwargs.keys(), permission, required)
    viewmeta._handle_allowed_attributes(
        _context, ('__call__', 'browserDefault', 'update', 'render',
                   'publishTraverse'), permission, required)

    # Register the interfaces.
    viewmeta._handle_for(_context, for_)

    # provide the custom provides interface if not allready provided
    if not provides.implementedBy(new_class):
        zope.interface.classImplements(new_class, provides)

    # Create the security checker for the new class
    zope.security.checker.defineChecker(
        new_class,
        zope.security.checker.Checker(required))

    # register pagelet
    _context.action(
        discriminator=('pagelet', for_, layer, name),
        callable=zope.component.zcml.handler,
        args=('registerAdapter',
              new_class, (for_, layer), provides, name, _context.info),)
